'use strict';
var express = require('express'),
    router = express.Router(),
    returnCode = require('../../utils/returnCode'),
    _ = require('lodash');
var mongoose = require('mongoose');
var ForumThread = mongoose.model('ForumThread');
var ForumComment = mongoose.model('ForumComment');

var forumThreadService = require('../../service/forumThreadService');
var forumRolePermissionService = require('../../service/forumRolePermissionService');
var forumPraiseLogService = require('../../service/forumPraiseLogService');
var forumShareLogService = require('../../service/forumShareLogService');
var forumCommentService = require('../../service/forumCommentService');
var forumUserThreadControlService = require('../../service/forumUserThreadControlService');
// var forumLimitActionRefService = require('../../service/forumLimitActionRefService');
var forumLimitOperationService = require('../../service/forumLimitOperationService');
var forumThreadAttendService = require('../../service/forumThreadAttendService');
var commentTips = require('../../utils/commentTips');
var httpService = require('../../service/httpService');
var forumAboutMEService = require('../../service/forumAboutMEService');
var user = require('../../utils/user');
var async = require('async');
module.exports = function(app) {
    app.use('/v1/forum', router);
};


function create(req, callback) {
    var entity = req.body;

    if (entity.pid == '') {
        entity.pid = null;
    }



    var address = entity.address || null;
    if (address) {
        entity.address = JSON.parse(address);
    }

    entity.ent_code = req.session.user.ent_code;
    entity.from = user.getMobileUser(req);

    //-----------处理前端上传图片开始
    //格式为['http://imageurl','http://imageurl'],
    //转换为[{urlFileName:'http://imageUrL'},{urlFileName:'http://imageUrL'}]
    var images = entity.images;
    if (images) {
        var array = [];
        for (var i = 0; i < images.length; i = i + 1) {
            array.push({
                urlFileName: images[i]
            });
        }
        entity.images = array;
    }
    //-----------处理前端上传图片结束
    if(entity.pid && entity.type == 3 ){
        forumThreadService.getByConditions({type:3,level:'2',pid:entity.pid,from:entity.from},function(err,doc){
            if(doc){
                var update = {
                    $set:{
                        images:entity.images,
                        content:entity.content
                    }
                };
                forumThreadService.updateThreadById(doc._id,update,function(err,updatedDoc){
                        callback(err,entity);
                });
            }else{
                forumThreadService.createThread(entity, function(err, entity) {
                    if (err) {
                        console.error(err);
                        callback(err, null);
                    } else {
                        callback(null, entity);
                    }
                });
            }
        });
    }else{
        forumThreadService.createThread(entity, function(err, entity) {
            if (err) {
                console.error(err);
                callback(err, null);
            } else {
                callback(null, entity);
            }
        });    
    }
}

//根据发帖者分页查询话题列表
router.get('/thread/searchThread', function(req, res, next) {

    //参数
    var pageNo = req.query.pageNo,
        pageSize = req.query.pageSize,
        from = req.session.mobileForumUser.userId,
        status = 1,
        ent_code = req.session.user.ent_code;


    if (pageNo && pageSize) {
        var q = {
            ent_code: ent_code,
            status: status,
            from: from
        };

        forumThreadService.findThreadByPage(pageNo, pageSize, q, function(err, result) {
            if (err) {
                console.error(err);
            }
            res.json(result);
        });
    } else {
        console.error('params error');
        res.json({
            result: false,
            err: 'params error'
        });
    }
});

//文章评论
router.get('/thread/searchComment', function(req, res, next) {
    
    //参数
    var pageNo = req.query.pageNo,
        pageSize = req.query.pageSize,
        from = req.session.mobileForumUser.userId,
        // status = 1,
        ent_code = req.session.user.ent_code;

    if (pageNo && pageSize) {
        var q = {
            ent_code: ent_code,
            // status: status,
            from:req.session.mobileForumUser.from
        };

       forumThreadService.findCommentByPage(pageNo, pageSize, q, function(err, result) {
            if (err) {
                console.error(err);
            }
            res.json(result);
        });
    } else {
        console.error('params error');
        res.json({
            result: false,
            err: 'params error'
        });
    }
});

//新增论坛文章
router.post('/thread/create', function(req, res, next) {
    var userId = req.session.mobileForumUser.userId;
    forumLimitOperationService.checkLimitOperationProhibitionOfSpeech(userId, function(err, isProhibition){
        if(err){
            console.error(err);
            res.json(returnCode.PROHIBITION_OF_SPEECH);
        }else{
            if(isProhibition){
                res.json(returnCode.PROHIBITION_OF_SPEECH);
            }else{
                var integral = req.session.openUser.integral || 0;
                var exp = req.session.openUser.exp || 0;
                forumRolePermissionService.checkRolePermiss(req.session.user.ent_code, 'create', integral, exp, function(err, result) {
                    if (err) {
                        console.error(err);
                        res.json(returnCode.BUSY);
                    } else {
                        if (result) { //有操作权限
                            create(req, function(err, thread) {
                                if (err) {
                                    console.error(err);
                                    res.json(returnCode.BUSY);
                                } else {
                                    forumAboutMEService.saveThread(thread);
                                    forumLimitOperationService.checkLimitOperationProhibitionAddIntegral(userId, function(err, flag){
                                        if(err){
                                            console.error(err);
                                        }else{
                                            if(!flag){
                                                httpService.sendRequest(req.session.user.ent_code, req.session.openUser.mid, 'post');
                                            }
                                        }
                                    });

                                    // httpService.sendRequest(req.session.user.ent_code, req.session.openUser.mid, 'post');
                                    var rs = {};
                                    rs.data = {
                                        'id': thread._id
                                    };
                                    res.json(_.assign(rs, returnCode.SUCCESS));
                                }
                            });
                        } else {
                            res.json(returnCode.ACTION_NOT_PERMISSION);
                        }
                    }
                });
            }
        }
    });
});

//获取目标论坛文章
router.get('/thread/:tid/get', function(req, res, next) {
    var tid = req.params.tid || null;
    if (tid) {

        async.waterfall([
            function(callback) {
                forumThreadService.updateThreadPvCount(tid, function(err, doc) {
                    if (err) {
                        callback(err, null);
                    } else {
                        callback(null, null);
                    }
                });
            },
            function(data, callback) {
                forumThreadService.getThreadById(tid, function(err, thread) {
                    //文章类型 1、文章 2、话题 3、照片墙
                    var info_id = thread.info._id,
                        pid = thread.pid,
                        source = req.session.mobileForumUser.source;
                    if (thread.type == 1 && thread.level == 1) {
                        httpService.createLog(req,source,info_id, 3, 1, tid);
                    } else if (thread.type == 1 && thread.level == 2) {
                        httpService.createLog(req,source,info_id, 3, 4, tid, pid);
                    } else if (thread.type == 2 && thread.level == 1) {
                        httpService.createLog(req,source,info_id, 3, 2, tid);
                    } else if (thread.type == 3 && thread.level == 1) {
                        httpService.createLog(req,source,info_id, 3, 3, tid);
                    }


                    if (err) {
                        callback(err, null);
                    } else {
                        callback(null, thread);
                    }
                });
            },
        ], function(err, thread) {
            if (err) {
                console.error(err);
                res.json(returnCode.BUSY);
            } else {
                var rs = {};
                rs.data = thread;
                res.json(_.assign(rs, returnCode.SUCCESS));
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//获取目标论坛文章
router.get('/thread/photo/:pid/get', function(req, res, next) {
    var pid = req.params.pid || null,
        userId = req.session.mobileForumUser.userId;;
    if (pid) {
        forumThreadService.getByConditions({type:3,level:'2',pid:pid,from:userId},function(err,thread){
            if (err) {
                console.error(err);
                res.json(returnCode.BUSY);
            } else {
                var rs = {};
                rs.data = thread;
                res.json(_.assign(rs, returnCode.SUCCESS));
            }    
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//更新文章状态、如：屏蔽
router.post('/thread/:tid/update', function(req, res, next) {
    var tid = req.params.tid;
    if (tid) {
        forumThreadService.updateThreadById(tid, req.body, function(err, thread) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                if(!isNaN(req.body.status)){
                    forumAboutMEService.updateThreadStatus(thread._id,thread.status)
                }
                res.json(returnCode.SUCCESS);
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//删除文章
router.post('/thread/:tid/delete', function(req, res, next) {
    var tid = req.params.tid;
    if (tid) {
        forumThreadService.deleteThreadById(tid, function(err, thread) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                res.json(returnCode.SUCCESS);
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//文章点赞
router.post('/thread/:tid/raise', function(req, res, next) {
    var userId = req.session.mobileForumUser.userId;
    var mid = req.session.openUser.mid;
    var tid = req.params.tid || null;
    if (tid) {
        var integral = req.session.openUser.integral || 0;
        var exp = req.session.openUser.exp || 0;

        //判断是否已经点赞
        forumPraiseLogService.queryPraiseLog(tid, null, user.getMobileUser(req), 1, function(err, logs) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                if (logs && logs.length > 0) {
                    res.json(returnCode.CAN_NOT_RAISE_REPEAT);
                } else {
                    //1.检查是否有权限
                    forumRolePermissionService.checkRolePermiss(req.session.user.ent_code, 'raise', integral, exp, function(err, result) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            if (result) { //有操作权限
                                //2.获取论坛文章
                                forumThreadService.getThreadById(tid, function(err, thread) {
                                    if (err) {
                                        res.json(returnCode.BUSY);
                                    } else {
                                        async.parallel([
                                            function(callback) {
                                                //3.更新文章统计数据(点赞数)
                                                if (thread) {
                                                    forumThreadService.updateThreadRaiseCount(tid, function(err, cb) {
                                                        if (err) {
                                                            callback(err, null);
                                                        } else {
                                                            callback(null, null);
                                                        }
                                                    });
                                                } else {
                                                    callback('cannot find thread by id', null);
                                                }
                                            },
                                            function(callback) {
                                                var entity = {
                                                    ent_code: req.session.user.ent_code,
                                                    user: user.getMobileUser(req),
                                                    thread: tid,
                                                    ip: req.ip,
                                                    comment: null,
                                                    type: 1
                                                };
                                                //4.创建点赞日志
                                                forumPraiseLogService.createPraiseLog(entity, function(err, result) {
                                                    if (err) {
                                                        callback(err, null);
                                                    } else {
                                                        callback(null, null);
                                                    }
                                                });
                                            }
                                        ], function(err, results) {
                                            if (err) {
                                                console.error(err);
                                                res.json(returnCode.BUSY);
                                            } else {

                                                forumLimitOperationService.checkLimitOperationProhibitionAddIntegral(userId, function(err, flag){
                                                    if(err){
                                                        console.error(err);
                                                    }else{
                                                        if(!flag){
                                                            httpService.sendRequest(req.session.user.ent_code, req.session.openUser.mid, 'thread_praise');
                                                        }
                                                    }
                                                });


                                                
                                                forumThreadService.getThreadById(tid, function(err, doc) {
                                                    if (err) {
                                                        console.error(err);
                                                        res.json(returnCode.BUSY);
                                                    } else {
                                                        var rs = {};
                                                        rs.praise_count = doc.praise_count;
                                                        res.json(_.assign(rs, returnCode.SUCCESS));
                                                    }
                                                });
                                            }
                                        });
                                    }
                                });
                            } else {
                                res.json(returnCode.ACTION_NOT_PERMISSION);
                            }
                        }
                    });
                }
            }
        });

    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//文章分享
router.post('/thread/:tid/share', function(req, res, next) {
    var tid = req.params.tid || null;
    if (tid) {
        var integral = req.session.openUser.integral || 0;
        var exp = req.session.openUser.exp || 0;
        var destination = req.body.destination || '1';
        //1.检查是否有权限
        forumRolePermissionService.checkRolePermiss(req.session.user.ent_code, 'share', integral, exp, function(err, result) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                if (result) { //有操作权限
                    //2.获取论坛文章
                    forumThreadService.getThreadById(tid, function(err, thread) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            async.parallel([
                                function(callback) {
                                    //3.更新文章统计数据(分享数)
                                    if (thread) {
                                        forumThreadService.updateThreadShareCount(tid, function(err, cb) {
                                            if (err) {
                                                callback(err, null);
                                            } else {
                                                callback(null, null);
                                            }
                                        });
                                    } else {
                                        callback('cannot find thread by id', null);
                                    }
                                },
                                function(callback) {
                                    var entity = {
                                        ent_code: req.session.user.ent_code,
                                        user: user.getMobileUser(req),
                                        thread: tid,
                                        destination: destination,
                                        ip: req.ip
                                    };
                                    //4.创建分享日志
                                    forumShareLogService.createShareLog(entity, function(err, result) {
                                        if (err) {
                                            callback(err, null);
                                        } else {
                                            callback(null, null);
                                        }
                                    });
                                }
                            ], function(err, results) {
                                if (err) {
                                    console.error(err);
                                    res.json(returnCode.BUSY);
                                } else {
                                    // httpService.sendRequest(req.session.user.ent_code,req.session.openUser.mid,'share');
                                    res.json(returnCode.SUCCESS);
                                }
                            });
                        }
                    });
                } else {
                    res.json(returnCode.ACTION_NOT_PERMISSION);
                }
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//新增文章评论
router.post('/thread/:tid/comment/add', function(req, res, next) {
    var userId = req.session.mobileForumUser.userId;
    forumLimitOperationService.checkLimitOperationProhibitionOfSpeech(userId, function(err, isProhibition){
        if(err){
            console.error(err);
            res.json(returnCode.PROHIBITION_OF_SPEECH);
        }else{
            if(isProhibition){
                res.json(returnCode.PROHIBITION_OF_SPEECH);
            }else{
                var tid = req.params.tid || null;
                if (tid) {
                    var integral = req.session.openUser.integral || 0;
                    var exp = req.session.openUser.exp || 0;
                    var destination = req.body.destination || '1';
                    //1.检查是否有权限
                    // forumRolePermissionService.checkRolePermiss(req.session.user.ent_code, 'comment', integral, exp, function(err, result) {
                    //     if (err) {
                    //         console.error(err);
                    //         res.json(returnCode.BUSY);
                    //     } else {
                            // if (result) { //有操作权限
                                var content = req.body.content;
                                if (content) { //评论不能为空
                                    //2.获取论坛文章
                                    forumThreadService.getThreadById(tid, function(err, thread) {
                                        if (err) {
                                            res.json(returnCode.BUSY);
                                        } else {
                                            async.parallel([
                                                function(callback) {
                                                    var floor = 1;
                                                    var entity = {
                                                        ent_code: req.session.user.ent_code,
                                                        from: user.getMobileUser(req),
                                                        content: content,
                                                        ip: req.ip,
                                                        thread: thread,
                                                        level: '1',
                                                        floor: floor
                                                    };

                                                    var conditions = {
                                                        thread: thread._id,
                                                        level: '1'
                                                    };
                                                    forumCommentService.count(conditions, function(err, count) {
                                                        if (err) {
                                                            callback(err, null);
                                                        } else {
                                                            // callback(null, count);
                                                            if(count){
                                                                entity.floor = count + 1;
                                                            }
                                                            //4.创建文章评论
                                                            forumCommentService.createComment(entity, function(err, newComment) {
                                                                if (err) {
                                                                    callback(err, null);
                                                                } else {
                                                                    try{
                                                                        if(user.getMobileUser(req).toString() != thread.from._id.toString()){//文章的作者不等于评论的作者才增加消息数
                                                                            commentTips.incrTips(req.session.user.ent_code,thread.from._id);
                                                                            forumThreadAttendService.threadAttend(thread._id,user.getMobileUser(req),function(err){});
                                                                        }
                                                                    }catch(e){
                                                                        console.log(e);
                                                                    }
                                                                    forumAboutMEService.saveCommentLevel1(thread,newComment);
                                                                    var comments = thread.comments;
                                                                    var array = [];
                                                                    if (comments && comments.items && comments.items.length > 0) {
                                                                        array = comments.items;
                                                                    }
                                                                    array.push(newComment._id);
                                                                    forumThreadService.updateThreadById(tid, {
                                                                        comments: array
                                                                    }, function(err, result) {
                                                                        if (err) {
                                                                            callback(err, newComment);
                                                                        } else {
                                                                            callback(null, newComment);
                                                                        }
                                                                    });
                                                                }
                                                            });
                                                        }
                                                    });
                                                },
                                                function(callback) {
                                                    //3.更新文章统计数据(评论数)
                                                    if (thread) {
                                                        forumThreadService.updateThreadCommentCountInc(tid, function(err, results) {
                                                            if (err) {
                                                                callback(err, null);
                                                            } else {
                                                                callback(null, null);
                                                            }
                                                        });
                                                    } else {
                                                        callback('cannot find thread by id', null);
                                                    }
                                                }
                                            ], function(err, results) {
                                                if (err) {
                                                    console.error(err);
                                                    res.json(returnCode.BUSY);
                                                } else {
                                                    forumLimitOperationService.checkLimitOperationProhibitionAddIntegral(userId, function(err, flag){
                                                        if(err){
                                                            console.error(err);
                                                        }else{
                                                            if(!flag){
                                                                httpService.sendRequest(req.session.user.ent_code, req.session.openUser.mid, 'comment');
                                                            }
                                                        }
                                                    });
                                                    forumCommentService.getPopulateCommentById(results[0]._id, function(err, c) {
                                                        if (err) {
                                                            res.json(returnCode.BUSY);
                                                        } else {
                                                            var rs = {};
                                                            rs.data = c;
                                                            res.json(_.assign(rs, returnCode.SUCCESS));
                                                        }
                                                    });

                                                }
                                            });
                                        }
                                    });
                                } else {
                                    res.json(returnCode.WRONG_PARAM);
                                }
                    //         } else {
                    //             res.json(returnCode.ACTION_NOT_PERMISSION);
                    //         }
                    //     }
                    // });
                } else {
                    res.json(returnCode.WRONG_PARAM);
                }
            }
        }
    });
});
//新增文章评论
router.post('/thread/:tid/comment/create', function(req, res, next) {
    var userId = req.session.mobileForumUser.userId;
    forumLimitOperationService.checkLimitOperationProhibitionOfSpeech(userId, function(err, isProhibition){
        if(err){
            console.error(err);
            res.json(returnCode.PROHIBITION_OF_SPEECH);
        }else{
            if(isProhibition){
                res.json(returnCode.PROHIBITION_OF_SPEECH);
            }else{
                var tid = req.params.tid || null;
                if (tid) {
                    var integral = req.session.openUser.integral || 0;
                    var exp = req.session.openUser.exp || 0;
                    var destination = req.body.destination || '1';
                    //1.检查是否有权限
                    forumRolePermissionService.checkRolePermiss(req.session.user.ent_code, 'comment', integral, exp, function(err, result) {
                        if (err) {
                            console.error(err);
                            res.json(returnCode.BUSY);
                        } else {
                            if (result) { //有操作权限
                                var content = req.body.content;
                                if (content) { //评论不能为空
                                    //2.获取论坛文章
                                    forumThreadService.getThreadById(tid, function(err, thread) {
                                        if (err) {
                                            res.json(returnCode.BUSY);
                                        } else {
                                            async.parallel([
                                                function(callback) {
                                                    var floor = 1;
                                                    var entity = {
                                                        ent_code: req.session.user.ent_code,
                                                        from: user.getMobileUser(req),
                                                        content: content,
                                                        ip: req.ip,
                                                        thread: thread,
                                                        level: '1',
                                                        floor: floor
                                                    };

                                                    var conditions = {
                                                        thread: thread._id,
                                                        level: '1'
                                                    };
                                                    forumCommentService.count(conditions, function(err, count) {
                                                        if (err) {
                                                            console.log(err);
                                                            callback(err, null);
                                                        } else {
                                                            // callback(null, count);
                                                            if(count){
                                                                entity.floor = count + 1;
                                                            }
                                                            //4.创建文章评论
                                                            forumCommentService.createComment(entity, function(err, newComment) {
                                                                if (err) {
                                                                    callback(err, null);
                                                                } else {
                                                                    try{
                                                                        if(user.getMobileUser(req).toString() != thread.from._id.toString()){//文章的作者不等于评论的作者才增加消息数
                                                                            commentTips.incrTips(req.session.user.ent_code,thread.from._id);
                                                                            forumThreadAttendService.threadAttend(thread._id,user.getMobileUser(req),function(err){});
                                                                        }
                                                                    }catch(e){
                                                                        console.log(e);
                                                                    }

                                                                    forumAboutMEService.saveCommentLevel1(thread,newComment);
                                                                    var comments = thread.comments;
                                                                    var array = [];
                                                                    if (comments && comments.items && comments.items.length > 0) {
                                                                        array = comments.items;
                                                                    }
                                                                    array.push(newComment._id);
                                                                    forumThreadService.updateThreadById(tid, {
                                                                        comments: array
                                                                    }, function(err, result) {
                                                                        if (err) {
                                                                            callback(err, null);
                                                                        } else {
                                                                            callback(null, null);
                                                                        }
                                                                    });
                                                                }
                                                            });
                                                        }
                                                    });
                                                },
                                                function(callback) {
                                                    //3.更新文章统计数据(评论数)
                                                    if (thread) {
                                                        forumThreadService.updateThreadCommentCountInc(tid, function(err, results) {
                                                            if (err) {
                                                                console.log(err);
                                                                callback(err, null);
                                                            } else {
                                                                callback(null, null);
                                                            }
                                                        });
                                                    } else {
                                                        callback('cannot find thread by id', null);
                                                    }
                                                }
                                            ], function(err, results) {
                                                if (err) {
                                                    console.error(err);
                                                    res.json(returnCode.BUSY);
                                                } else {
                                                    forumLimitOperationService.checkLimitOperationProhibitionAddIntegral(userId, function(err, flag){
                                                        if(err){
                                                            console.error(err);
                                                        }else{
                                                            if(!flag){
                                                                httpService.sendRequest(req.session.user.ent_code, req.session.openUser.mid, 'comment');
                                                            }
                                                        }
                                                    });

                                                    async.parallel([
                                                        function(callback) {
                                                            //获取最新5条评论
                                                            var conditions = {
                                                                thread: tid,
                                                                level: '1'
                                                            };
                                                            forumCommentService.getAllComment(conditions, 1, 5, function(err, results) {
                                                                if (err) {
                                                                    console.log(err);
                                                                    callback(err, null);
                                                                } else {
                                                                    callback(null, results);
                                                                }
                                                            });
                                                        },
                                                        function(callback) {
                                                            forumThreadService.getThreadById(tid, function(err, thread) {
                                                                if (err) {
                                                                    console.log(err);
                                                                    callback(err, null);
                                                                } else {
                                                                    callback(null, thread);
                                                                }
                                                            });
                                                        }
                                                    ], function(err, results) {
                                                        if (err) {
                                                            res.json(returnCode.BUSY);
                                                        } else {
                                                            var rs = {};
                                                            rs.comments = results[0].items;
                                                            rs.commentCount = results[1].comment_count;
                                                            res.json(_.assign(rs, returnCode.SUCCESS));
                                                        }
                                                    });

                                                }
                                            });
                                        }
                                    });
                                } else {
                                    res.json(returnCode.WRONG_PARAM);
                                }
                            } else {
                                res.json(returnCode.ACTION_NOT_PERMISSION);
                            }
                        }
                    });
                } else {
                    res.json(returnCode.WRONG_PARAM);
                }
            }
        }
    });
});
//新建文章评论的子评论
router.post('/thread/:tid/comment/:cid/create', function(req, res, next) {
    var userId = req.session.mobileForumUser.userId;
    forumLimitOperationService.checkLimitOperationProhibitionOfSpeech(userId, function(err, isProhibition){
        if(err){
            console.error(err);
            res.json(returnCode.PROHIBITION_OF_SPEECH);
        }else{
            if(isProhibition){
                res.json(returnCode.PROHIBITION_OF_SPEECH);
            }else{
                var tid = req.params.tid || null;
                var cid = req.params.cid || null;
                if (tid && cid) {
                    var integral = req.session.openUser.integral || 0;
                    var exp = req.session.openUser.exp || 0;
                    var destination = req.body.destination || '1';
                    req.body.from = user.getMobileUser(req);

                    //1.检查是否有权限
                    forumRolePermissionService.checkRolePermiss(req.session.user.ent_code, 'reply', integral, exp, function(err, result) {
                        if (err) {
                            console.error(err);
                            res.json(returnCode.BUSY);
                        } else {
                            if (result) { //有操作权限
                                var content = req.body.content;
                                if (content) { //评论不能为空
                                    //2.获取论坛评论
                                    forumCommentService.getCommentById(cid, function(err, comment) {
                                        if (err) {
                                            res.json(returnCode.BUSY);
                                        } else {
                                            async.parallel([
                                                function(callback) {
                                                    var entity = {
                                                        ent_code: req.session.user.ent_code,
                                                        from: user.getMobileUser(req),
                                                        to: req.body.to,
                                                        content: content,
                                                        ip: req.ip,
                                                        thread: tid,
                                                        level: '2'
                                                    };
                                                    //4.创建文章评论
                                                    forumCommentService.createComment(entity, function(err, newComment) {
                                                        if (err) {
                                                            callback(err, null);
                                                        } else {
                                                            try{
                                                                if(user.getMobileUser(req).toString() != req.body.to.toString()){//被评论人 和评论人不是同一个
                                                                    commentTips.incrTips(req.session.user.ent_code,req.body.to);
                                                                    forumThreadAttendService.threadAttend(tid,user.getMobileUser(req),function(err){});
                                                                }
                                                            }catch(e){
                                                                  console.log(e);
                                                            }
                                                            forumAboutMEService.saveCommentLevel2(tid,comment,newComment);
                                                            var comments = comment.comments;
                                                            var array = [];
                                                            if (comments && comments.length > 0) {
                                                                array = comments;
                                                            }
                                                            array.push(newComment._id);
                                                            forumCommentService.updateCommentById(cid, {
                                                                comments: array
                                                            }, function(err, result) {
                                                                if (err) {
                                                                    callback(err, null);
                                                                } else {
                                                                    callback(null, newComment);
                                                                }
                                                            });
                                                        }
                                                    });
                                                },
                                                function(callback) {
                                                    forumCommentService.updateCommentCount(cid, function(err, results) {
                                                        if (err) {
                                                            console.log(err);
                                                            callback(err, null);
                                                        } else {
                                                            callback(null, null);
                                                        }
                                                    });
                                                }
                                            ], function(err, results) {
                                                if (err) {
                                                    console.error(err);
                                                    res.json(returnCode.BUSY);
                                                } else {
                                                    forumLimitOperationService.checkLimitOperationProhibitionAddIntegral(userId, function(err, flag){
                                                        if(err){
                                                            console.error(err);
                                                        }else{
                                                            if(!flag){
                                                                httpService.sendRequest(req.session.user.ent_code, req.session.openUser.mid, 'reply');
                                                            }
                                                        }
                                                    });
                                                    forumCommentService.getPopulateCommentById(results[0]._id, function(err, c) {
                                                        if (err) {
                                                            res.json(returnCode.BUSY);
                                                        } else {

                                                            var rs = {};
                                                            rs.data = c;
                                                            res.json(_.assign(rs, returnCode.SUCCESS));
                                                        }
                                                    });
                                                }
                                            });
                                        }
                                    });
                                } else {
                                    res.json(returnCode.WRONG_PARAM);
                                }
                            } else {
                                res.json(returnCode.ACTION_NOT_PERMISSION);
                            }
                        }
                    });
                } else {
                    res.json(returnCode.WRONG_PARAM);
                }
            }
        }
    });
});
//更改评论状态为3(删除)
router.post('/thread/:tid/comment/:cid/disable', function(req, res, next) {
    var user_id = user.getMobileUser(req),
        tid = req.params.tid,
        cid = req.params.cid;
    if (tid && cid) {
        forumCommentService.getCommentById(cid,function(err,comment){
            if(comment &&  user_id){
                if(comment.from.toString() == user_id.toString()){
                    forumCommentService.changeStatus(cid,2,function(err,update){
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            if(comment && comment.level==1){
                                forumAboutMEService.updateCommentLevel1Status(comment._id,2);
                            }
                            if(comment && comment.level==2){
                                forumAboutMEService.updateCommentLevel2Status(comment._id,2);
                            }
                            res.json(returnCode.SUCCESS);
                        }
                    });
                }else{
                    res.json(returnCode.ACTION_NOT_PERMISSION);
                }    
            }else{
                res.json(returnCode.WRONG_PARAM);
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//删除文章评论
router.post('/thread/:tid/comment/:cid/delete', function(req, res, next) {
    var tid = req.params.tid || null;
    var cid = req.params.cid || null;
    if (tid && cid) {
        forumCommentService.getCommentById(cid, function(err, result) {
            if (err) {
                console.error(err);
                res.json(returnCode.BUSY);
            } else {
                if (result) {
                    if (result.from === user.getMobileUser(req)) {
                        async.parallel([
                            function(callback) {
                                forumCommentService.deleteCommentById(cid, function(err, result) {
                                    if (err) {
                                        callback(err, null);
                                    } else {
                                        callback(null, null);
                                    }
                                });
                            }
                        ], function(err, results) {
                            if (err) {
                                res.json(returnCode.BUSY);
                            } else {
                                res.json(returnCode.SUCCESS);
                            }
                        });

                    } else {
                        res.json(returnCode.ACTION_NOT_PERMISSION);
                    }
                } else {
                    res.json(returnCode.WRONG_PARAM);
                }
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//评论列表
router.get('/thread/:tid/comment/list', function(req, res, next) {

	var tid=req.params.tid || null;
	var pageNo = req.query.pageNo || 1;
    var tid = req.params.tid || null;
    var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;

    // var conditions = {
    //     ent_code: req.session.user.ent_code,
    //     thread: tid,
    //     level: '1',
    //     status:1
    // };

    var conditions = {
        ent_code: req.session.user.ent_code,
        thread: tid,
        level: '1'
    };

    if (tid) {
        //获取最新5条评论                                          
        forumCommentService.getAllComment(conditions, pageNo, pageSize, function(err, results) {
            if (err) {
                console.log(err);
                res.json(returnCode.BUSY);
            } else {
                res.json(_.assign(results, returnCode.SUCCESS));
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

router.get('/thread/:tid/comment/list/byFloor', function(req, res, next) {

    var tid = req.params.tid || null;
    var floor = req.query.floor || 1;

    // var conditions = {
    //     ent_code: req.session.user.ent_code,
    //     thread: tid,
    //     level: '1',
    //     status:1
    // };

    var countConditions = {
        ent_code: req.session.user.ent_code,
        thread: tid,
        level: '1'
    };

    var conditions = {
        ent_code: req.session.user.ent_code,
        thread: tid,
        level: '1',
        floor: {
            '$gte': floor - 9,
            '$lte': floor
        }
    };

    //conditions.floor['$lte'] = 12;

    if (tid) {
        //获取最新5条评论              
        forumCommentService.count(countConditions, function(err, count) {
            if (err) {
                console.log(err);
                res.json(returnCode.BUSY);
            } else {
                forumCommentService.getAllComment(conditions, 1, 10, function(err, results) {
                    if (err) {
                        console.log(err);
                        res.json(returnCode.BUSY);
                    } else {
                        results.total = count;
                        res.json(_.assign(results, returnCode.SUCCESS));
                    }
                });
            }

        });

    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});


//评论点赞
router.post('/thread/:tid/comment/:cid/raise', function(req, res, next) {
    var userId = req.session.mobileForumUser.userId;
    var tid = req.params.tid || null;
    var cid = req.params.cid || null;
    var mid = req.session.openUser.mid;
    if (tid && cid) {

        var integral = req.session.openUser.integral || 0;
        var exp = req.session.openUser.exp || 0;
        //判断是否已经点赞
        forumPraiseLogService.queryPraiseLog(tid, cid, user.getMobileUser(req), 2, function(err, logs) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                if (logs && logs.length > 0) {
                    res.json(returnCode.CAN_NOT_RAISE_REPEAT);
                } else {
                    //1.检查是否有权限
                    forumRolePermissionService.checkRolePermiss(req.session.user.ent_code, 'raise', integral, exp, function(err, result) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            if (result) { //有操作权限
                                //2.获取论坛文章
                                forumThreadService.getThreadById(tid, function(err, thread) {
                                    if (err) {
                                        res.json(returnCode.BUSY);
                                    } else {
                                        async.parallel([
                                            function(callback) {
                                                forumCommentService.updatePraiseCount(cid, function(err, results) {
                                                    if (err) {
                                                        console.log(err);
                                                        callback(err, null);
                                                    } else {
                                                        callback(null, null);
                                                    }
                                                });
                                            },
                                            function(callback) {
                                                var entity = {
                                                    ent_code: req.session.user.ent_code,
                                                    user: user.getMobileUser(req),
                                                    thread: tid,
                                                    ip: req.ip,
                                                    comment: cid,
                                                    type: 2
                                                };
                                                //4.创建点赞日志
                                                forumPraiseLogService.createPraiseLog(entity, function(err, result) {
                                                    if (err) {
                                                        callback(err, null);
                                                    } else {
                                                        callback(null, null);
                                                    }
                                                });
                                            }
                                        ], function(err, results) {
                                            if (err) {
                                                console.error(err);
                                                res.json(returnCode.BUSY);
                                            } else {

                                                forumLimitOperationService.checkLimitOperationProhibitionAddIntegral(userId, function(err, flag){
                                                    if(err){
                                                        console.error(err);
                                                    }else{
                                                        if(!flag){
                                                            httpService.sendRequest(req.session.user.ent_code, req.session.openUser.mid, 'comment_praise');
                                                        }
                                                    }
                                                });

                                                
                                                //返回点赞总数
                                                forumCommentService.getCommentById(cid, function(err, doc) {
                                                    if (err) {
                                                        console.error(err);
                                                        res.json(returnCode.BUSY);
                                                    } else {
                                                        var rs = {};
                                                        rs.praise_count = doc.praise_count;
                                                        res.json(_.assign(rs, returnCode.SUCCESS));
                                                    }
                                                });

                                            }
                                        });
                                    }
                                });
                            } else {
                                res.json(returnCode.ACTION_NOT_PERMISSION);
                            }
                        }
                    });
                }
            }
        });

    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//用户屏蔽文章
router.post('/thread/:tid/userShield', function(req, res, next) {
    var tid = req.params.tid || null;
    if (tid) {
        forumUserThreadControlService.getUserThreadControlById(user.getMobileUser(req), function(err, doc) {
            if (err) {
                res.json(returnCode.BUSY);
            } else {
                if (doc) {
                    var thread = doc.thread;
                    thread.push(tid);
                    forumUserThreadControlService.updateUserThreadControlById(doc._id, {
                        thread: thread
                    }, function(err, result) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            res.json(returnCode.SUCCESS);
                        }
                    });
                } else {
                    var array = [tid];
                    var entity = {
                        ent_code: req.session.user.ent_code,
                        user: user.getMobileUser(req),
                        thread: array
                    };
                    forumUserThreadControlService.createUserThreadControl(entity, function(err, result) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            res.json(returnCode.SUCCESS);
                        }
                    });
                }
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//获取论坛话题列表
router.get('/thread/:tid/topics', function(req, res, next) {
    var tid = req.params.tid || null;
    var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;

    var conditions = {
        ent_code: req.session.user.ent_code,
        pid: tid,
        type: 2,
        status:{$ne:3}
    };

    if (tid) {

        async.waterfall([
            function(callback) {
                forumUserThreadControlService.getUserThreadControlById(user.getMobileUser(req), function(err, doc) {
                    if (err) {
                        callback(err, null);
                    } else {
                        if (doc) {
                            callback(null, doc);
                        } else {
                            callback(null, null);
                        }
                    }
                });
            }
        ], function(err, result) {
            if (err) {
                console.error(err);
                res.json(returnCode.BUSY);
            } else {
                if (result) {
                    conditions._id = {
                        $nin: result.thread
                    };
                }
                //获取子话题数据
                forumThreadService.getAllThreadByFid(conditions, pageNo, pageSize, null, function(err, results) {
                    if (err) {
                        console.error(err);
                        res.json(returnCode.BUSY);
                    } else {
                        res.json(_.assign(results, returnCode.SUCCESS));
                    }
                });
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//获取论坛最新照片墙列表
router.get('/thread/:tid/latestPhotos', function(req, res, next) {
    var tid = req.params.tid || null;
    var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;

    var conditions = {
        ent_code: req.session.user.ent_code,
        pid: tid,
        type: 3,
        status:{$ne:3},
        images: {$exists: true, $not: {$size: 0}} 
    };

    if (tid) {
        async.waterfall([
            function(callback) {
                forumUserThreadControlService.getUserThreadControlById(user.getMobileUser(req), function(err, doc) {
                    if (err) {
                        callback(err, null);
                    } else {
                        if (doc) {
                            callback(null, doc);
                        } else {
                            callback(null, null);
                        }
                    }
                });
            }
        ], function(err, result) {
            if (err) {
                console.error(err);
                res.json(returnCode.BUSY);
            } else {
                if (result) {
                    conditions._id = {
                        $nin: result.thread
                    };
                }
                //获取子话题数据
                forumThreadService.getAllThreadByFid(conditions, pageNo, pageSize, '-created', function(err, results) {
                    if (err) {
                        console.error(err);
                        res.json(returnCode.BUSY);
                    } else {
                        res.json(_.assign(results, returnCode.SUCCESS));
                    }
                });
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});

//获取论坛最热照片墙列表
router.get('/thread/:tid/hotPhotos', function(req, res, next) {
    var tid = req.params.tid || null;
    var pageNo = req.query.pageNo || 1;
    var pageSize = req.query.pageSize || 10;

    var conditions = {
        ent_code: req.session.user.ent_code,
        pid: tid,
        type: 3,
        status:{$ne:3},
        images: {$exists: true, $not: {$size: 0}} 
    };

    if (tid) {
        async.waterfall([
            function(callback) {
                forumUserThreadControlService.getUserThreadControlById(user.getMobileUser(req), function(err, doc) {
                    if (err) {
                        callback(err, null);
                    } else {
                        if (doc) {
                            callback(null, doc);
                        } else {
                            callback(null, null);
                        }
                    }
                });
            }
        ], function(err, result) {
            if (err) {
                console.error(err);
                res.json(returnCode.BUSY);
            } else {
                if (result) {
                    conditions._id = {
                        $nin: result.thread
                    };
                }
                //获取子话题数据
                forumThreadService.getAllThreadByFid(conditions, pageNo, pageSize, '-praise_count', function(err, results) {
                    if (err) {
                        console.error(err);
                        res.json(returnCode.BUSY);
                    } else {
                        res.json(_.assign(results, returnCode.SUCCESS));
                    }
                });
            }
        });
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});


//逻辑删除文章
router.post('/thread/:tid/disable', function(req, res, next) {
    var user_id = user.getMobileUser(req);
    var tid = req.params.tid;
    if (tid) {
        forumThreadService.getThreadById(tid,function(err,thread){
            if(thread.from && user_id){
                if(thread.from._id.toString() == user_id.toString()){
                    forumThreadService.logicDeleteThreadById(tid, function(err, flag) {
                        if (err) {
                            res.json(returnCode.BUSY);
                        } else {
                            forumAboutMEService.updateThreadStatus(tid,3);
                            res.json(returnCode.SUCCESS);
                        }
                    });
                }else{
                    res.json(returnCode.ACTION_NOT_PERMISSION);
                }
            }else{
                res.json(returnCode.WRONG_PARAM);
            }
        })
    } else {
        res.json(returnCode.WRONG_PARAM);
    }
});
